<?php

namespace common\logic\Distribution\Distribution;

use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistribution;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistributionBuilder;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistributionRepository;
use common\logic\Distribution\ProductDistribution\ProductDistribution;
use common\logic\Distribution\ProductDistribution\ProductDistributionBuilder;
use common\logic\Distribution\ProductDistribution\ProductDistributionRepository;
use common\logic\Order\Order\Order;
use common\logic\Order\Order\OrderRepository;
use common\logic\Order\ProductOrder\ProductOrderBuilder;
use common\logic\PointSale\PointSale\PointSale;
use common\logic\Producer\Producer\Producer;
use common\logic\Product\Product\Product;
use common\logic\Product\Product\ProductRepository;
use common\logic\Subscription\Subscription\Subscription;
use common\logic\User\UserProducer\UserProducer;
use common\logic\User\UserProducer\UserProducerRepository;

class DistributionBuilder extends BaseService implements BuilderInterface
{
    protected DistributionRepository $distributionRepository;
    protected DistributionSolver $distributionSolver;
    protected PointSaleDistributionRepository $pointSaleDistributionRepository;
    protected PointSaleDistributionBuilder $pointSaleDistributionBuilder;
    protected ProductRepository $productRepository;
    protected ProductDistributionRepository $productDistributionRepository;
    protected ProductDistributionBuilder $productDistributionBuilder;
    protected OrderRepository $orderRepository;
    protected UserProducerRepository $userProducerRepository;
    protected ProductOrderBuilder $productOrderBuilder;

    public function __construct()
    {
        $this->distributionRepository = $this->loadService(DistributionRepository::class);
        $this->distributionSolver = $this->loadService(DistributionSolver::class);
        $this->pointSaleDistributionRepository = $this->loadService(PointSaleDistributionRepository::class);
        $this->pointSaleDistributionBuilder = $this->loadService(PointSaleDistributionBuilder::class);
        $this->productRepository = $this->loadService(ProductRepository::class);
        $this->productDistributionRepository = $this->loadService(ProductDistributionRepository::class);
        $this->orderRepository = $this->loadService(OrderRepository::class);
        $this->userProducerRepository = $this->loadService(UserProducerRepository::class);
        $this->productOrderBuilder = $this->loadService(ProductOrderBuilder::class);
    }

    public function instanciateDistribution(Producer $producer, string $date, bool $delivery = true): Distribution
    {
        $distribution = new Distribution();

        $distribution->id_producer = $producer->id;
        $distribution->date = $date;
        $distribution->delivery = (int) $delivery;

        return $distribution;
    }

    // initDistribution
    public function createDistribution(Producer $producer, string $date, bool $delivery = true): Distribution
    {
        $distribution = $this->instanciateDistribution($producer, $date, $delivery);
        $distribution->save();

        $this->initPointSaleDistribution($distribution);
        $this->initProductDistribution($distribution);

        return $distribution;
    }

    public function createDistributionIfNotExist(Producer $producer, string $date, int $delivery = 1)
    {
        $distribution = $this->distributionRepository->getOne($producer, $date);

        if (!$distribution) {
            $distribution = $this->createDistribution($producer, $date, 1);
        }

        return $distribution;
    }

    public function initPointSaleDistribution(Distribution $distribution): void
    {
        $countPointSaleDistribution = $this->pointSaleDistributionRepository->countByDistribution($distribution);
        if (!$countPointSaleDistribution) {
            $this->pointSaleDistributionBuilder->createAll($distribution, true);
        }
    }

    public function initProductDistribution(Distribution $distribution): void
    {
        $productArray = $this->productRepository->get();
        $countProductDistribution = $this->productDistributionRepository->countByDistribution($distribution);

        if (!$countProductDistribution) {
            foreach ($productArray as $product) {
                $this->linkProduct($distribution, $product);
            }
        }
    }

    /**
     * Lie un produit à une distribution.
     */
    public function linkProduct(Distribution $distribution, Product $product): ProductDistribution
    {
        $productDistribution = $this->productDistributionBuilder->createIfNotExist($distribution, $product);
        $this->updateOrderProductPrices($distribution, $product);

        return $productDistribution;
    }

    /**
     * Lie un produit aux distributions futures.
     */
    public function linkProductIncomingDistributions(Product $product): void
    {
        $distributionArray = $this->distributionRepository->getIncoming();

        foreach ($distributionArray as $distribution) {
            $this->linkProduct($distribution, $product);
        }
    }

    /**
     * Lie un point de vente à une distribution
     */
    public function linkPointSale(Distribution $distribution, PointSale $pointSale): PointSaleDistribution
    {
        $pointSaleDistribution = $this->pointSaleDistributionBuilder->createIfNotExist($distribution, $pointSale);
        $dayString = $this->distributionSolver->getDayAsString($distribution);

        $pointSaleDistribution->delivery = 0;
        $deliveryDay = 'delivery_' . $dayString;
        if ($pointSale->$deliveryDay) {
            $pointSaleDistribution->delivery = 1;
        }

        $pointSaleDistribution->save();

        return $pointSaleDistribution;
    }

    /**
     * Lie un point de vente aux distributions futures.
     */
    public function linkPointSaleIncomingDistributions(PointSale $pointSale): void
    {
        $distributionArray = $this->distributionRepository->getIncoming();

        foreach ($distributionArray as $distribution) {
            $this->linkPointSale($distribution, $pointSale);
        }
    }

    public function updateOrderProductPrices(Distribution $distribution, Product $product): void
    {
        $ordersArray = $this->orderRepository->getByDistribution($distribution, 'AND origin != "user"');

        if ($ordersArray) {
            foreach ($ordersArray as $order) {
                foreach ($order->productOrder as $productOrder) {
                    if ($productOrder->id_product == $product->id) {

                        $user = null;
                        $userProducer = null;
                        if (isset($order->user) && $order->user) {
                            $user = $order->user;
                            $userProducer = $this->userProducerRepository->getOne($user, $distribution->producer);
                        }

                        $this->productOrderBuilder->updatePrice(
                            $productOrder,
                            $user,
                            $userProducer,
                            $order->pointSale,
                            $productOrder->quantity
                        );
                    }
                }
            }
        }
    }

    /**
     * Active ou désactive la distribution.
     */
    public function active(Distribution $distribution, bool $active = true): void
    {
        $this->pointSaleDistributionBuilder->createAll($distribution, true);

        $distribution->active = (int) $active;
        $distribution->save();

        if ($active) {
            Subscription::addAll($distribution->date);
        }
    }
}