<?php

namespace common\logic\Order\Order;

use common\logic\BaseService;
use common\logic\Distribution\Distribution\Distribution;
use common\logic\Order\ProductOrder\ProductOrder;
use common\logic\Order\ProductOrder\ProductOrderRepository;
use common\logic\Producer\Producer\Producer;
use common\logic\Producer\Producer\ProducerRepository;
use common\logic\Product\Product\Product;
use common\logic\RepositoryInterface;
use common\logic\User\User\User;
use yii\helpers\Html;

class OrderRepository extends BaseService implements RepositoryInterface
{
    protected ProductOrderRepository $productOrderRepository;
    protected ProducerRepository $producerRepository;
    protected OrderSolver $orderSolver;

    public function __construct()
    {
        $this->productOrderRepository = $this->loadService(ProductOrderRepository::class);
        $this->producerRepository = $this->loadService(ProducerRepository::class);
        $this->orderSolver = $this->loadService(OrderSolver::class);
    }

    public function defaultOptionsSearch(): array
    {
        return [
            'with' => [
                'productOrder',
                'productOrder.product',
                'creditHistory',
                'creditHistory.userAction',
                'pointSale'
            ],
            'join_with' => ['distribution', 'user', 'user.userProducer'],
            'orderby' => 'order.date ASC',
            'attribute_id_producer' => 'distribution.id_producer'
        ];
    }

    public function findOneById(int $id)
    {
        return Order::searchOne(['order.id' => $id]);;
    }

    /**
     * Recherche et initialise des commandes.
     */
    // searchBy
    public function findBy(array $params = [], array $options = []): array
    {
        $orders = Order::searchBy($params, $options);

        /*
         * Initialisation des commandes
         */

        if (is_array($orders)) {
            if (count($orders)) {
                foreach ($orders as $order) {
                    if (is_a($order, 'common\logic\Order\Order\Order')) {
                        $order->init();
                    }
                }
                return $orders;
            }
        } else {
            $order = $orders;
            if (is_a($order, 'common\logic\Order\Order\Order')) {
                return $order->init();
            } // count
            else {
                return $order;
            }
        }

        return false;
    }

    public function findByDistribution(Distribution $distribution, string $conditionAppend = '')
    {
        return Order::searchAll([
            'distribution.date' => $distribution->date,
            'distribution.id_producer' => $distribution->id_producer
        ],
            [
                'conditions' => 'date_delete IS NULL ' . $conditionAppend
            ]);
    }
    
    public function queryHistory(Producer $producer, User $user, string $type = 'incoming')
    {
        $query = Order::find()
            ->with('productOrder', 'pointSale', 'creditHistory')
            ->joinWith('distribution', 'distribution.producer')
            ->where([
                'id_user' => $user->id,
                'distribution.id_producer' => $producer->id
            ])
            ->params([':date_today' => date('Y-m-d')]);

        $queryIncoming = clone $query;
        $queryIncoming
            ->andWhere('distribution.date >= :date_today')
            ->orderBy('distribution.date ASC');

        $queryPassed = clone $query;
        $queryPassed
            ->andWhere('distribution.date < :date_today')
            ->orderBy('distribution.date DESC');

        return ['incoming' => $queryIncoming, 'passed' => $queryPassed];
    }

    /**
     * Retourne le résumé du panier au format HTML.
     * @TODO : à déplacer dans OrderSolver
     */
    public function getCartSummary(Order $order, $htmlFormat = true): string
    {
        if (!isset($order->productOrder)) {
            $order->productOrder = $this->productOrderRepository->getByOrder($order);
        }

        $html = '';
        $i = 0;
        $count = count($order->productOrder);
        foreach ($order->productOrder as $p) {
            if (isset($p->product)) {
                $html .= Html::encode($p->product->name) . ' (' . $p->quantity . '&nbsp;' . Product::strUnit(
                        $p->unit,
                        'wording_short',
                        true
                    ) . ')';
                if (++$i != $count) {
                    if ($htmlFormat) {
                        $html .= '<br />';
                    } else {
                        $html .= "\n";
                    }
                }
            }
        }

        return $html;
    }

    /**
     * Retourne le résumé du paiement (montant, statut).
     * @TODO : à déplacer dans OrderSolver
     */
    public function getAmountSummary(Order $order): string
    {
        $html = '';
        $creditActive = $this->producerRepository->getConfig('credit');
        $html .= $this->orderSolver->getAmountWithTax($order, Order::AMOUNT_TOTAL, true);

        if ($creditActive) {
            $html .= '<br />';
            if ($order->paid_amount) {
                if ($this->orderSolver->getPaymentStatus($order) == Order::PAYMENT_PAID) {
                    $html .= '<span class="label label-success">Payée</span>';
                } elseif ($this->orderSolver->getPaymentStatus($order) == Order::PAYMENT_UNPAID) {
                    $html .= '<span class="label label-danger">Non payée</span><br />
                                                Reste <strong>' . $this->orderSolver->getAmount(
                            $order,
                            Order::AMOUNT_REMAINING,
                            true
                        ) . '</strong> à payer';
                } elseif ($this->orderSolver->getPaymentStatus($order) == Order::PAYMENT_SURPLUS) {
                    $html .= '<span class="label label-success">Payée</span>';
                }
            } else {
                $html .= '<span class="label label-default">Non réglé</span>';
            }
        }

        return $html;
    }

    /**
     * Retourne l'état de la commande (livrée, modifiable ou en préparation.
     * @TODO : à déplacer dans OrderSolver
     */
    public function getState(Order $order): string
    {
        $orderDate = strtotime($order->distribution->date);
        $today = strtotime(date('Y-m-d'));
        $todayHour = date('G');
        $dayDistribution = strtolower(date('l', strtotime($order->distribution->date)));

        $orderDelay = $this->producerRepository->getConfig(
            'order_delay',
            $order->distribution->id_producer
        );
        $orderDelaySpecific = $this->producerRepository->getConfig(
            'order_delay_' . $dayDistribution,
            $order->distribution->id_producer
        );
        if ($orderDelaySpecific) {
            $orderDelay = $orderDelaySpecific;
        }

        $orderDeadline = $this->producerRepository->getConfig(
            'order_deadline',
            $order->distribution->id_producer
        );
        $orderDeadlineSpecific = $this->producerRepository->getConfig(
            'order_deadline_' . $dayDistribution,
            $order->distribution->id_producer
        );
        if ($orderDeadlineSpecific) {
            $orderDeadline = $orderDeadlineSpecific;
        }

        $nbDays = (int) round((($orderDate - $today) / (24 * 60 * 60)));

        if ($nbDays <= 0) {
            return Order::STATE_DELIVERED;
        } elseif ($nbDays >= $orderDelay &&
            ($nbDays != $orderDelay ||
                ($nbDays == $orderDelay && $todayHour < $orderDeadline))) {
            return Order::STATE_OPEN;
        }

        return Order::STATE_PREPARATION;
    }

    public function findOneLastOfYear(Producer $producer)
    {
        return Order::find()->innerJoinWith('distribution', true)
            ->where(['>=', 'distribution.date', date('Y') . '-01-01'])
            ->andWhere([
                'distribution.id_producer' => $producer->id
            ])
            ->andWhere(['not', ['order.reference' => null]])
            ->orderBy('order.reference DESC')
            ->one();
    }


}